	LIST    P = 16C74,  F = INHX8M, n = 66
;
;******************************************************************************
;
; This program implements a real time clock using the TMR1 module of the 
; PIC16Cxx family.
;
;       Program = CAPT_2.ASM
;       Revision Date:   7-19-94
;
;******************************************************************************
; 
;
; HARDWARE SETUP
;
;           CCP2 Compare Output
;           CCP1 Capture Input
;           CCP2  --->  CCP1
;
;
            INCLUDE <C74_reg.h>
            INCLUDE <CAPT.h>
;
;
PICMaster       EQU     TRUE         ; A Debugging Flag
Debug           EQU     TRUE         ; A Debugging Flag
Debug_PU        EQU     TRUE         ; A Debugging Flag
;
;
; Reset address. Determine type of RESET
;
        org     RESET_V                 ; RESET vector location
RESET       BSF     STATUS, RP0         ; Bank 1
            BTFSC   PCON, POR           ; Power-up reset?
            GOTO    START               ; YES
            GOTO    OTHER_RESET         ; NO, a WDT or MCLR reset
;
; This is the Periperal Interrupt routine. Need to determine the type
; of interrupt that occurred. The following interrupts are enabled:
;   1.  CCP1 Capture Occured
;   2.  CCP2 Compare Occured
;
    page
        org     ISR_V              ; Interrupt vector location
PER_INT_V	
            BCF     STATUS, RP0    ; Bank 0
            BTFSC   PIR1, CCP1IF   ; CCP1 Interrupt occured? (Capture)
            GOTO    CAPTURE        ; YES, Service the CCP1 Interrupt
            BTFSC   PIR2, CCP2IF   ; CCP2 Interrupt occured? (Compare)
            GOTO    COMPARE        ; YES, Service the CCP2 Interrupt
            BTFSC   PIR1, TMR1IF   ; NO, Timer 1 Overflow?
            GOTO    T1OVFL         ; YES,
ERROR1                             ; NO, Error Condition - Unknown Interrupt
            BSF     PORTD, 1       ; Toggle a PORT pin
            BCF     PORTD, 1
            GOTO    ERROR1
;
ERROR2                             ; NO, Error Condition - Unknown Interrupt
            BSF     PORTD, 2       ; Toggle a PORT pin
            BCF     PORTD, 2
            GOTO    ERROR2
;
; The Compare generates a Square wave based on the value on PORTB (in DUMMY_PB)
; and on PORTD (in DUMMY_PD). PORTB is loaded into low compare latch and PORTD
; is loaded into the high compare latch. If the value of the ports is not changed,
; a capture overflow condition will occur when PORTD:PORTB > 7Fh. This overflow
; is only indicated by the time between captures being much less then expected.
;
COMPARE
            BCF     PIR2, CCP2IF        ; Clear CCP2 Interrupt Flag
        if ( PICMaster )
            MOVF    DUMMY_PB, W         ;
        else
            MOVF    PORTB, W            ;
        endif
            ADDWF   CCPR2L, F           ; Update Compare register pair latch
            BTFSC   STATUS, C           ;
            INCF    CCPR2H, F           ;
        if ( PICMaster )
            MOVF    DUMMY_PD, W         ;
        else
            MOVF    PORTD, W            ;
        endif
            ADDWF   CCPR2H, F           ;
            INCF    CCP2_INT_CNT        ;
            BSF     CCP2CON, 0          ; On Compare match, CCP2 pin = L
            BTFSS   CCP2_INT_CNT, 0     ;
            BCF     CCP2CON, 0          ; On Compare match, CCP2 pin = H
END_COMPARE
            RETFIE                      ; Return / Enable Global Interrupts

;
; The result of the new capture minus the old capture is stored in the new capture
; registers (CAPT_NEW_H:CAPT_NEW_L)
;
CAPTURE
            BCF     PIR1, CCP1IF    ; Clear CCP1 Interrupt Flag
            MOVF    CCPR1L, W       ; New capture value (low byte)
            MOVWF   CAPT_NEW_L      ;
            MOVF    CCPR1H, W       ; New capture value (high byte)
            MOVWF   CAPT_NEW_H      ;
;
            MOVF    CAPT_OLD_L, W   ;
            SUBWF   CAPT_NEW_L, F   ; Subtract the low bytes of the 2 captures
            BTFSS   STATUS, C       ; Did a borrow occur?
            DECF    CAPT_NEW_H, F   ; YES, Decrement old capture (high byte)
            MOVF    CAPT_OLD_H, W   ; New capture value (low byte)
            SUBWF   CAPT_NEW_H, F   ; Subtract the low bytes of the 2 captures
LOAD_OLD    MOVF    CCPR1L, W       ; New capture value (low byte)
            MOVWF   CAPT_OLD_L      ;
            MOVF    CCPR1H, W       ; New capture value (high byte)
            MOVWF   CAPT_OLD_H      ;
END_CAPTURE
            RETFIE
;
;
T1OVFL
            BCF     PIR1, TMR1IF    ; Clear T1 Overflow Interrupt Flag
            RETFIE                  ; Return / Enable Global Interrupts
;
;
;******************************************************************************
;*****      Start program here, Power-On Reset occurred.
;******************************************************************************
;
START                               ; POWER_ON Reset (Beginning of program)
            BCF     STATUS, RP0     ; Bank 0
            CLRF    TMR1H           ; 
            CLRF    TMR1L           ;
;
MCLR_RESET                          ; A Master Clear Reset
            BCF     STATUS, RP0     ; Bank 0
            CLRF    STATUS          ; Do initialization (Bank 0)
            CLRF    INTCON
            CLRF    PIR1
            BSF     STATUS, RP0     ; Bank 1
            MOVLW   0x00            ; The LCD module does not like to work w/ weak pull-ups
            MOVWF   OPTION_R        ;
            CLRF    PIE1            ; Disable all peripheral interrupts
            CLRF    PIE2            ; Disable all peripheral interrupts
            MOVLW   0xFF            ;
            MOVWF   ADCON1          ; Port A is Digital.
;
;
            BCF     STATUS, RP0     ; Bank 0
            CLRF    PORTA           ; ALL PORT output should output Low.
            CLRF    PORTB
            CLRF    PORTC
            CLRF    PORTD
            CLRF    PORTE
            BCF     T1CON, TMR1ON   ; Timer 1 is NOT incrementing
;
            BSF     STATUS, RP0     ; Select Bank 1
            CLRF    TRISA           ; RA5 -  0 outputs
            MOVLW   0xFF            ;
            MOVWF   TRISB           ; RB7 - 0 inputs 
            CLRF    TRISC           ; RC Port are outputs
            BSF     TRISC, 2        ; CCP1 is an INPUT
            MOVWF   TRISD           ; RD Port are inputs
            CLRF    TRISE           ; RE Port are outputs
            BSF     PIE1, CCP1IE    ; Enable CCP1 Interrupt
            BSF     PIE2, CCP2IE    ; Enable CCP2 Interrupt
            BCF     STATUS, RP0     ; Select Bank 0
;
;
; Initialize the Special Function Registers (SFR) interrupts
;
            CLRF    PIR1            ;
            CLRF    PIR2            ;
            CLRF    T1CON           ; Timer mode
            BSF     INTCON, PEIE    ; Enable Peripheral Interrupts
            BSF     INTCON, GIE     ; Enable all Interrupts
;
; Set-up timer and compare latches and then turn timer1 on.
;
            BCF     T1CON, TMR1ON       ; Turn OFF timer1
        if ( PICMaster )
            MOVF    DUMMY_PB, W         ;
        else
            MOVF    PORTB, W            ;
        endif
            ADDWF   CCPR2L, F           ; Update Compare register pair latch
            BTFSC   STATUS, C           ;
            INCF    CCPR2H, F           ;
        if ( PICMaster )
            MOVF    DUMMY_PD            ;
        else
            MOVF    PORTD, W            ;
        endif
            ADDWF   CCPR2H, F           ;
            MOVLW   0x08                ; On match CCP2 = H level
            MOVWF   CCP2CON             ;
            MOVLW   0x05                ; Capture on every rising edge
            MOVWF   CCP1CON             ;
            BSF     T1CON, TMR1ON       ; Turn ON timer1
;
;
;
lzz         goto    lzz                 ; Loop waiting for interrupts (for use with PICMASTER)
;
;
; Here is where you do things depending on the type of RESET (Not a Power-On Reset).
;
OTHER_RESET   BTFSS  STATUS, TO     ; WDT Time-out?
WDT_TIMEOUT   GOTO   ERROR1         ; YES, This is error condition
        if ( Debug_PU )
            goto   START            ; MCLR reset, Goto START
        else
            GOTO   MCLR_RESET       ; MCLR reset, Goto MCLR_RESET
        endif
;
        if (Debug )
END_START     NOP                   ; END lable for debug
        endif
;
;
    org     PMEM_END            ; End of Program Memory
            GOTO    ERROR1          ; If you get here your program was lost

    end

