	LIST    P = 16C74,  F = INHX8M, n = 66
;
;******************************************************************************
;
; This program outputs a pulse train on the CCP1 pin, as specified by the
; values in the CCPR1H:CCPR1L.
;                 
;                     ____          ____      __
;  Pulse Train    ___|    |________|    |____|
;  Data Value         <---- 1 ----> <-- 0 -->
;                       ^      ^      ^    ^
;                       |      |      |    |
;                       |  T_ONE_BIT  |  T_ZERO_BIT
;                       |             |
;                   PULSE_TIME    PULSE_TIME
;
;
;       Program = COMP_1.ASM
;       Revision Date:   7-13-94
;
;******************************************************************************
; 
;
; HARDWARE SETUP
;    PORTB        -  Data to serial transmit on CCP pin 
;
;
            INCLUDE <C74_reg.h>
            INCLUDE <COMP.h>
;
PICMaster       EQU     TRUE        ; A Debugging Flag
Debug           EQU     TRUE        ; A Debugging Flag
Debug_PU        EQU     TRUE         ; A Debugging Flag
;
;
; Reset address. Determine type of RESET
;
        org     RESET_V                 ; RESET vector location
RESET       BSF     STATUS, RP0         ; Bank 1
            BTFSC   PCON, POR           ; Power-up reset?
            GOTO    START               ; YES
            GOTO    OTHER_RESET         ; NO, a WDT or MCLR reset
;
; This is the Periperal Interrupt routine. Need to determine the type
; of interrupt that occurred. The following interrupts are enabled:
;   1.  CCP Capture Occured
;
    page
        org     ISR_V              ; Interrupt vector location
PER_INT_V	
        if ( Debug )
            bsf     PORTA, 0       ; Turn on strobe
        endif                      ;
            BCF     STATUS, RP0    ; Bank 0
            BTFSC   PIR1, CCP1IF   ; Compare Interrupt occured?
            GOTO    CCP1_INT       ; YES, Service the TMR1 Interrupt
ERROR1                             ; NO, Error Condition - Unknown Interrupt
            BSF     PORTA, 2       ; Toggle a PORT pin
            BCF     PORTA, 2
            GOTO    ERROR1
;
ERROR2                             ; NO, Error Condition - Unknown Interrupt
            BSF     PORTA, 3       ; Toggle a PORT pin
            BCF     PORTA, 3
            GOTO    ERROR2
;
;
;******************************************************************************
; In the CCP interrupt.
; Since timer1 is not cleared on a CCP match, the value in the
; CCPR1H:CCPR1L register pair must be updated. This is done with
; a 16-bit add. Also after the 1st CCP1 match (CCP1 pin goes high)
; the next match will force it low. Depending on the value of the data bit
; determines the value add to the CCPR1H:CCPR1L register pair.
;
; After the data has been transmitted, the pin will have a sync pulse and
; then remain low for 300 us.
;******************************************************************************
;

CCP1_INT
            BCF     PIR1, CCP1IF     ; Clear CCP1 Interrupt Flag
            INCF    CCP1_INT_CNT     ;
            BTFSS   CCP1_INT_CNT, 0  ;
            GOTO    SYNC_PULSE
DATA_PULSE
            DECF    DATA_CNT          ; Decrement the Count of data bits
            BTFSC   STATUS, Z         ; Have we transmitted all the Data Bits?
            GOTO    PERIOD_DELTA      ; YES, Delay to 300 us
            RLF     XMIT_DATA, F      ; NO, get next bit to transmit
            BTFSC   STATUS, C         ; Is the bit to transmit a '1'?
            GOTO    ONE_DATA          ; YES, Stay low for 17.6 us
ZERO_DATA
            MOVLW   LOW ( T_ZERO_BIT )  ; NO, Stay low for 8.8 us
            ADDWF   CCPR1L, F           ; Update Compare register pair latch
            BTFSC   STATUS, C           ;
            INCF    CCPR1H, F           ;
            MOVLW   HIGH (T_ZERO_BIT )  ;
            ADDWF   CCPR1H, F           ;
            GOTO    RET_FIE
;
ONE_DATA
            MOVLW   LOW ( T_ONE_BIT )   ; Stay low for 17.6 us
            ADDWF   CCPR1L, F           ; Update Compare register pair latch
            BTFSC   STATUS, C           ;
            INCF    CCPR1H, F           ;
            MOVLW   HIGH ( T_ONE_BIT )  ;
            ADDWF   CCPR1H, F           ;
            INCF    ONES_CNT            ; Increment the number of '1's in the byte
            GOTO    RET_FIE
;
PERIOD_DELTA
            MOVF    ONES_CNT, W    ;
            ANDLW   0x0F           ; Only want 9 states (0 1s to 8 1s)
            ADDWF   PCL, F         ;
            GOTO    ZERO_1         ; There was 0 ones in the data byte
            GOTO    ONE_1          ; There was 1 one in the data byte
            GOTO    TWO_1          ; There was 2 ones in the data byte
            GOTO    THREE_1        ; There was 3 ones in the data byte
            GOTO    FOUR_1         ; There was 4 ones in the data byte
            GOTO    FIVE_1         ; There was 5 ones in the data byte
            GOTO    SIX_1          ; There was 6 ones in the data byte
            GOTO    SEVEN_1        ; There was 7 ones in the data byte
            GOTO    EIGHT_1        ; There was 8 ones in the data byte
;
SYNC_PULSE
            MOVLW   LOW ( PULSE_TIME )  ; Update Compare register pair latch
            ADDWF   CCPR1L, F           ;
            BTFSC   STATUS, C           ;
            INCF    CCPR1H, F           ;
            MOVLW   HIGH ( PULSE_TIME ) ;
            ADDWF   CCPR1H, F           ;
            BSF     CCP1CON, 0          ; On Compare match, CCP1 pin = L
            RETFIE
;
ZERO_1
            MOVLW   LOW ( ZERO_1S )   ; Update Compare register pair latch
            ADDWF   CCPR1L, F         ;
            BTFSC   STATUS, C         ;
            INCF    CCPR1H, F         ;
            MOVLW   HIGH ( ZERO_1S )  ;
            ADDWF   CCPR1H, F         ;
            GOTO    RET_FIE
;
ONE_1
            MOVLW   LOW ( ONE_1S )    ; Update Compare register pair latch
            ADDWF   CCPR1L, F         ;
            BTFSC   STATUS, C         ;
            INCF    CCPR1H, F         ;
            MOVLW   HIGH ( ONE_1S )   ;
            ADDWF   CCPR1H, F         ;
            GOTO    RET_FIE
;
TWO_1
            MOVLW   LOW ( TWO_1S )    ; Update Compare register pair latch
            ADDWF   CCPR1L, F         ;
            BTFSC   STATUS, C         ;
            INCF    CCPR1H, F         ;
            MOVLW   HIGH ( TWO_1S )   ;
            ADDWF   CCPR1H, F         ;
            GOTO    RET_FIE
;
THREE_1
            MOVLW   LOW ( THREE_1S )  ; Update Compare register pair latch
            ADDWF   CCPR1L, F         ;
            BTFSC   STATUS, C         ;
            INCF    CCPR1H, F         ;
            MOVLW   HIGH ( THREE_1S ) ;
            ADDWF   CCPR1H, F         ;
            GOTO    RET_FIE
;
FOUR_1
            MOVLW   LOW ( FOUR_1S )   ; Update Compare register pair latch
            ADDWF   CCPR1L, F         ;
            BTFSC   STATUS, C         ;
            INCF    CCPR1H, F         ;
            MOVLW   HIGH ( FOUR_1S )  ;
            ADDWF   CCPR1H, F         ;
            GOTO    RET_FIE
;
FIVE_1
            MOVLW   LOW ( FIVE_1S )   ; Update Compare register pair latch
            ADDWF   CCPR1L, F         ;
            BTFSC   STATUS, C         ;
            INCF    CCPR1H, F         ;
            MOVLW   HIGH ( FIVE_1S )  ;
            ADDWF   CCPR1H, F         ;
            GOTO    RET_FIE
;
SIX_1
            MOVLW   LOW ( SIX_1S )    ; Update Compare register pair latch
            ADDWF   CCPR1L, F         ;
            BTFSC   STATUS, C         ;
            INCF    CCPR1H, F         ;
            MOVLW   HIGH ( SIX_1S )   ;
            ADDWF   CCPR1H, F         ;
            GOTO    RET_FIE
;
SEVEN_1
            MOVLW   LOW ( SEVEN_1S )   ; Update Compare register pair latch
            ADDWF   CCPR1L, F         ;
            BTFSC   STATUS, C         ;
            INCF    CCPR1H, F         ;
            MOVLW   HIGH ( SEVEN_1S ) ;
            ADDWF   CCPR1H, F         ;
            GOTO    RET_FIE
;
EIGHT_1
            MOVLW   LOW ( EIGHT_1S )   ; Update Compare register pair latch
            ADDWF   CCPR1L, F         ;
            BTFSC   STATUS, C         ;
            INCF    CCPR1H, F         ;
            MOVLW   HIGH ( EIGHT_1S ) ;
            ADDWF   CCPR1H, F         ;
            GOTO    RET_FIE

RET_FIE
            BCF     CCP1CON, 0        ; On Compare match, CCP1 pin = H
            RETFIE                    ; Return / Enable Global Interrupts
;
    page
;
;******************************************************************************
;*****      Start program here, Power-On Reset occurred.
;******************************************************************************
;
START                               ; POWER_ON Reset (Beginning of program)
            BCF     STATUS, RP0     ; Bank 0
            CLRF    TMR1H           ; 
            CLRF    TMR1L           ;
;
MCLR_RESET                          ; A Master Clear Reset
            BCF     STATUS, RP0     ; Bank 0
            CLRF    STATUS          ; Do initialization (Bank 0)
            CLRF    INTCON
            CLRF    PIR1
            BSF     STATUS, RP0     ; Bank 1
            MOVLW   0x80            ; Disable PORTB weak pull-ups
            MOVWF   OPTION_R        ;
            CLRF    PIE1            ; Disable all peripheral interrupts
            MOVLW   0xFF            ;
            MOVWF   ADCON1          ; Port A is Digital.
;
;
            BCF     STATUS, RP0     ; Bank 0
            CLRF    PORTA           ; ALL PORT output should output Low.
            CLRF    PORTB
            CLRF    PORTC
            CLRF    PORTD
            CLRF    PORTE
            BCF     T1CON, TMR1ON   ; Timer 1 is NOT incrementing
;
            BSF     STATUS, RP0     ; Select Bank 1
            CLRF    TRISA           ; RA5 -  0 outputs
            MOVLW   0xFF            ;
            MOVWF   TRISB           ; RB Port are inputs 
            CLRF    TRISC           ; RC Port are outputs
            CLRF    TRISD           ; RD Port are outputs
            CLRF    TRISE           ; RE Port are outputs
            BSF     PIE1, CCP1IE    ; Enable CCP1 Interrupt
            BCF     STATUS, RP0     ; Select Bank 0
;
    page
;
;
; Initialize the Special Function Registers (SFR) interrupts
;
            CLRF    PIR1            ;
            CLRF    T1CON           ; Timer mode
            BSF     INTCON, PEIE    ; Enable Peripheral Interrupts
            BSF     INTCON, GIE     ; Enable all Interrupts
;
; Set-up timer and compare latches and then turn timer1 on.
;
            BCF     T1CON, TMR1ON  ; Turn OFF timer1
            MOVLW   CCPREG_HI      ; TMR1 = CCPR1H:CCPR1L - 1
            MOVWF   TMR1H          ;
            MOVLW   CCPREG_LO      ;
            MOVWF   TMR1L          ;
            DECF    TMR1L          ;
            BTFSC   STATUS, C      ;
            DECF    TMR1H          ;
            MOVLW   0x08           ; On match CCP1 = H level
            MOVWF   CCP1CON        ;
            MOVLW   0x09           ;
            MOVWF   DATA_CNT       ; 8-bits to transfer
            CLRF    ONES_CNT       ; Result after xmit holds the number of 1's in a byte
            MOVLW   0xFF           ;
            MOVWF   CCP1_INT_CNT   ; No CCP1 transmit interrups yet
            BSF     T1CON, TMR1ON  ; Turn ON timer1
;
;
; This code segment is an infinite loop that will always transmit the data
; contained in the XMIT_DATA register. After each byte is transmitted a new
; byte is read. If using PICMASTER (in stand alone mode), this is read from 
; a register that is updated after a break (at NOP). If in a system, PORTB
; is read. All other variables are reinitalized after each byte.
;
NEXT_BYTE
;
WAIT        MOVF    DATA_CNT, w    ;
            BTFSS   STATUS, Z      ; Is DATA_CNT = 0 ?
            GOTO    WAIT           ; NO, must wait until YES
            NOP                    ;
        if ( Debug )
            bcf     PORTA, 0       ; Turn off strobe
        endif                      ;

        if ( PICMaster )
            MOVF    DUMMY_PB, W    ;
        else
            MOVF    PORTB, W       ;
        endif
            MOVWF   XMIT_DATA      ; New data to transmit
            MOVLW   0xFF           ; 
            MOVWF   CCP1_INT_CNT   ;
            MOVLW   0x09           ; 
            MOVWF   DATA_CNT       ;
            CLRF    ONES_CNT       ;
            GOTO    NEXT_BYTE      ;
;
;
; Here is where you do things depending on the type of RESET (Not a Power-On Reset).
;
OTHER_RESET   BTFSS  STATUS, TO     ; WDT Time-out?
WDT_TIMEOUT   GOTO   ERROR1         ; YES, This is error condition
        if ( Debug_PU )
            goto   START            ; MCLR reset, Goto START
        else
            GOTO   MCLR_RESET       ; MCLR reset, Goto MCLR_RESET
        endif
;
        if (Debug )
END_START     NOP                   ; END lable for debug
        endif
;
;
    org     PMEM_END            ; End of Program Memory
            GOTO    ERROR1          ; If you get here your program was lost

    end


