	LIST    P = 16C74,  F = INHX8M, n = 66
;
;******************************************************************************
;
; This program outputs a PWM signal on the CCP1 pin. The duty cycle and period
; of the PWM is read every time TMR1 overflows.
;    PERIOD     = PORTB
;    DUTY CYCLE = PORTD and PORTE<1:0>
;
; The prescaler of TMR2 is selected by the state of PORTA<1:0> after reset
;     RA1:RA0    Prescaler multiplies Tcyc by
;       0 0       1
;       0 1       4
;       1 x      16
;
;
;       Program = PWM_1.ASM
;       Revision Date:   7-13-94
;
;******************************************************************************
; 
;
; HARDWARE SETUP
;    PORTA<1:0>   -  Prescaler to TMR2, read only after reset
;    PORTB        -  Period of PWM 
;    PORTD        -  Duty Cycle high of PWM (8-bits)
;    PORTE<1:0>   -  Duty Cycle low of PWM (2-bits)
;
;
            INCLUDE <C74_reg.h>
            INCLUDE <PWM.h>
;
PICMaster       EQU     TRUE         ; A Debugging Flag
Debug           EQU     TRUE         ; A Debugging Flag
Debug_PU        EQU     TRUE         ; A Debugging Flag
;
;
; Reset address. Determine type of RESET
;
        org     RESET_V                 ; RESET vector location
RESET       BSF     STATUS, RP0         ; Bank 1
            BTFSC   PCON, POR           ; Power-up reset?
            GOTO    START               ; YES
            GOTO    OTHER_RESET         ; NO, a WDT or MCLR reset
;
; This is the Periperal Interrupt routine. Need to determine the type
; of interrupt that occurred. The following interrupts are enabled:
;   1.  CCP Capture Occured
;
    page
        org     ISR_V              ; Interrupt vector location
PER_INT_V	
            BCF     STATUS, RP0    ; Bank 0
            BTFSC   PIR1, TMR1IF   ; TMR1 Overflow Interrupt occured?
            GOTO    T1OVFL         ; YES, Service the TMR1 Interrupt
ERROR1                             ; NO, Error Condition - Unknown Interrupt
            BSF     PORTA, 2       ; Toggle a PORT pin
            BCF     PORTA, 2
            GOTO    ERROR1
;
ERROR2                             ; NO, Error Condition - Unknown Interrupt
            BSF     PORTA, 3       ; Toggle a PORT pin
            BCF     PORTA, 3
            GOTO    ERROR2
;
T1OVFL
            BCF     PIR1, TMR1IF    ; Clear T1 Overflow Interrupt Flag
        if (PICMaster )
            MOVF    DUMMY_PD, W     ;
        else
            MOVF    PORTD, W        ;
        endif
            MOVWF   DC_HI           ;
        if (PICMaster )
            MOVF    DUMMY_PE, W     ;
        else
            MOVF    PORTE, W        ;
        endif
            MOVWF   DC_LO           ;
        if (PICMaster )
            MOVF    DUMMY_PB, W     ;
        else
            MOVF    PORTB, W        ;
        endif
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   T2_PERIOD       ;
            BCF     STATUS, RP0     ; Bank 0
;
WAIT_DC
            MOVF    TMR2, W         ; Read present TMR2 register value
            SUBWF   PR2, W          ; How close is the timer to rolling over
            ANDLW   0x0F            ; Does this make it zero?
            BTFSC   STATUS, Z       ; If Z is set, near rollover
            GOTO    WAIT_DC         ; loop until rolled over
            MOVF    DC_HI, W        ; else losd the duty cycle values
            MOVWF   CCPR1L          ; Load DC high
            MOVLW   0x0F            ;
            ANDWF   CCP1CON, F      ; Set the DC low bits
            BTFSC   DC_LO, 1        ;
            BSF     CCP1CON, CCP1X  ;
            BTFSC   DC_LO, 0        ;
            BSF     CCP1CON, CCP1Y  ;
			BCF     PIR1, TMR2IF    ; Clear the TRM2 = PR2 flag
;
WAIT_PR
            BTFSS   PIR1, TMR2IF    ; LOOP waiting for TRM2 = PR2
            GOTO    WAIT_PR         ; Need to wait until TMR2 = PR2 so that
                                    ;    Duty Cycle is latched
            BSF     STATUS, RP0     ; Bank 1
            MOVLW   0x0F            ; Load TMR2 period with minimum value Fh
            MOVWF   PR2             ; 
            MOVLW   0xF0            ;
            ANDWF   T2_PERIOD, W    ; Determine if period needs to be greater
            BTFSC   STATUS, Z       ;
            GOTO    NO_OFFSET       ; NO, Period is the minimum
PR_OFFSET
            MOVLW   0x0F            ; Yes, calculate additional offset
            SUBWF   T2_PERIOD, W    ;
            ADDWF   PR2, F          ; ADD Period offset
;
NO_OFFSET
            BCF     STATUS, RP0     ; Bank 0
            RETFIE                  ; Return / Enable Global Interrupts
;
    page
;
;******************************************************************************
;*****      Start program here, Power-On Reset occurred.
;******************************************************************************
;
START                               ; POWER_ON Reset (Beginning of program)
            BCF     STATUS, RP0     ; Bank 0
            CLRF    TMR1H           ; 
            CLRF    TMR1L           ;
;
MCLR_RESET                          ; A Master Clear Reset
            CLRF    STATUS          ; Do initialization (Bank 0)
            CLRF    INTCON
            CLRF    PIR1
            BSF     STATUS, RP0     ; Bank 1
            MOVLW   0x80            ; 
            MOVWF   OPTION_R        ;
            CLRF    PIE1            ; Disable all peripheral interrupts
            MOVLW   0xFF            ;
            MOVWF   ADCON1          ; Port A is Digital.
;
;
            BCF     STATUS, RP0     ; Bank 0
            CLRF    PORTA           ; ALL PORT output should output Low.
            CLRF    PORTB
            CLRF    PORTC
            CLRF    PORTD
            CLRF    PORTE
;
            BSF     STATUS, RP0     ; Select Bank 1
            MOVLW   0xFF            ;
            MOVWF   TRISA           ; RA5 -  0 inputs
            MOVWF   TRISB           ; RB7 - 0 inputs 
            CLRF    TRISC           ; RC Port are outputs
            MOVWF   TRISD           ; RD Port are inputs
            MOVWF   TRISE           ; RE Port are inputs
            MOVWF   PR2             ; Default PWM period
            BSF     PIE1, TMR1IE    ; Enable TMR1 Interrupt
            BCF     STATUS, RP0     ; Select Bank 0
;
            MOVLW   0X0C            ; CCP module is in 
            MOVWF   CCP1CON         ; PWM output mode
;
; Initialize the Special Function Registers (SFR) interrupts
;
            CLRF    PIR1            ;
            CLRF    T1CON           ;
            CLRF    T2CON           ;
        if (PICMaster )
            BTFSC   DUMMY_PA, 0     ;
        else
            BTFSC   PORTA, 0        ;
        endif
            BSF     T2CON, 0        ;
;
        if (PICMaster )
            BTFSC   DUMMY_PA, 1     ;
        else
            BTFSC   PORTA, 1        ;
        endif
            BSF     T2CON, 1        ;
;
            BSF     INTCON, PEIE    ; Enable Peripheral Interrupts
            BSF     INTCON, GIE     ; Enable all Interrupts
            BSF     T1CON, TMR1ON   ; Turn Timer 1 ON
            BSF     T2CON, TMR2ON   ; Turn Timer 2 ON
;
lzz         goto    lzz             ; Loop waiting for TMR1 interrupt
;
; Here is where you do things depending on the type of RESET (Not a Power-On Reset).
;
OTHER_RESET   BTFSS  STATUS, TO     ; WDT Time-out?
WDT_TIMEOUT   GOTO   ERROR1         ; YES, This is error condition
        if ( Debug_PU )
            goto   START            ; MCLR reset, Goto START
        else
            GOTO   MCLR_RESET       ; MCLR reset, Goto MCLR_RESET
        endif
;
        if (Debug )
END_START     NOP                   ; END lable for debug
        endif
;
;
    org     PMEM_END            ; End of Program Memory
            GOTO    ERROR1          ; If you get here your program was lost

    end

